﻿namespace UltraPivotGridFlatDataSource
{
    using System;
    using System.Collections.Generic;
    using System.Windows.Forms;
    using Infragistics.Olap;
    using Infragistics.Olap.FlatData;
    using Infragistics.Win.UltraWinPivotGrid;

    public partial class Form1 : Form
    {
        private const int DATA_COUNT = 100;
        private List<CityData> cities;
        private List<string> sports;
        readonly Random random = new Random();
        private FlatDataSource ds;

        public Form1()
        {
            this.InitializeComponent();
            
            this.olapDataSelector1.MeasuresNodeText = "Data Selector Measures' Custom Text";
            this.olapDataSelector1.RootNodeText = "Data Selector Root Node Custom Text";

            this.WindowState = FormWindowState.Maximized;
            ds = this.InitializePivotData();

            ds.DataSourceError += Ds_DataSourceError;
            ds.InitializeAsyncCompleted += Ds_InitializeAsyncCompleted;
            this.ultraPivotGrid1.DataSource = ds;
            this.ultraPivotGrid1.RowHeaderLayout = RowHeaderLayout.Compact;
            this.olapDataSelector1.DataSource = ds;
        }

        private FlatDataSource InitializePivotData()
        {
            IEnumerable<SportsData> itemSource = this.InitSportsData();
            var settings = new FlatDataSourceInitialSettings
            {
                Rows = "[SportsData].[Nation]",
                Columns = "[SportsData].[Sport]",
                Measures = "[Measures].[Budget]"
            };

            var ds = new FlatDataSource(itemSource, typeof(SportsData), settings);
            ds.InitializeMeasureDescriptor += Ds_InitializeMeasureDescriptor;
            ds.InitializeHierarchyDescriptor += Ds_InitializeHierarchyDescriptor;
            ds.AggregateMeasure += Ds_AggregateMeasure;
            var parameters = new CubeGenerationParameters();
            ds.GenerateCube(parameters);
            ds.InitializeAsync(ultraPivotGrid1);

            return ds;
        }

        private IEnumerable<SportsData> InitSportsData()
        {
            this.InitCities();
            this.InitSports();

            var items = new List<SportsData>();
            for (var i = 0; i < DATA_COUNT; i++)
            {
                int a = this.random.Next(1, 10);
                var dt = DateTime.Today.AddDays(a);

                var cityNation = this.cities[this.random.Next(this.cities.Count)];

                var newItem = new SportsData
                {
                    Sport = this.sports[this.random.Next(this.sports.Count)],
                    Nation = cityNation.Nation,
                    City = cityNation.City,
                    Budget = this.random.Next(1000, 1000000) / 100,
                    Players = this.random.Next(1, 100),
                    Date = dt
                };
                items.Add(newItem);
            }

            return items;
        }

        private void InitCities()
        {
            if (this.cities != null)
            {
                return;
            }

            this.cities = new List<CityData>
                {
                    new CityData {City = "Los Angeles", Nation = "United States"},
                    new CityData {City = "San Diego", Nation = "United States"},
                    new CityData {City = "San Francisco", Nation = "United States"},
                    new CityData {City = "Detroit", Nation = "United States"},
                    new CityData {City = "Miami", Nation = "United States"},
                    new CityData {City = "Fort Lauderdale", Nation = "United States"},
                    new CityData {City = "New York", Nation = "United States"},
                    new CityData {City = "Pittsburgh", Nation = "United States"},
                    new CityData {City = "Philadelphia", Nation = "United States"},
                    new CityData {City = "Chicago", Nation = "United States"},
                    new CityData {City = "Denver", Nation = "United States"},
                    new CityData {City = "Minneapolis", Nation = "United States"},
                    new CityData {City = "Kansas City", Nation = "United States"},
                    new CityData {City = "Memphis", Nation = "United States"},
                    new CityData {City = "New Orleans", Nation = "United States"},
                    new CityData {City = "Dallas", Nation = "United States"},
                    new CityData {City = "Houston", Nation = "United States"},
                    new CityData {City = "Washington D.C.", Nation = "United States"},
                    new CityData {City = "Seattle", Nation = "United States"},
                    new CityData {City = "Portland", Nation = "United States"},
                    new CityData {City = "Phoenix", Nation = "United States"},
                    new CityData {City = "Tuscon", Nation = "United States"},
                    new CityData {City = "Salt Lake City", Nation = "United States"},
                    new CityData {City = "Boston", Nation = "United States"},
                    new CityData {City = "Cheyenne", Nation = "United States"},
                    new CityData {City = "Santa Fe", Nation = "United States"},
                    new CityData {City = "Virginia Beach", Nation = "United States"},
                    new CityData {City = "London", Nation = "United Kingdom"},
                    new CityData {City = "Manchester", Nation = "United Kingdom"},
                    new CityData {City = "Newcastle", Nation = "United Kingdom"},
                    new CityData {City = "Belfast", Nation = "United Kingdom"},
                    new CityData {City = "Cardiff", Nation = "United Kingdom"},
                    new CityData {City = "Birmingham", Nation = "United Kingdom"},
                    new CityData {City = "Glasgow", Nation = "United Kingdom"},
                    new CityData {City = "Dublin", Nation = "Ireland"},
                    new CityData {City = "Cork", Nation = "Ireland"},
                    new CityData {City = "Galway", Nation = "Ireland"},
                    new CityData {City = "Paris", Nation = "France"},
                    new CityData {City = "Milan", Nation = "Italy"},
                    new CityData {City = "Venice", Nation = "Italy"},
                    new CityData {City = "Barcelona", Nation = "Spain"},
                    new CityData {City = "Seville", Nation = "Spain"},
                    new CityData {City = "Madrid", Nation = "Spain"},
                    new CityData {City = "Berlin", Nation = "Germany"},
                    new CityData {City = "Munich", Nation = "Germany"},
                    new CityData {City = "Dusseldorf", Nation = "Germany"},
                    new CityData {City = "Cologne", Nation = "Germany"},
                    new CityData {City = "Frankfurt", Nation = "Germany"},
                    new CityData {City = "Warsaw", Nation = "Poland"},
                    new CityData {City = "Athens", Nation = "Greece"},
                    new CityData {City = "Sophia", Nation = "Bulgaria"},
                    new CityData {City = "Amsterdam", Nation = "Netherlands"},
                    new CityData {City = "Bruges", Nation = "Belgium"},
                    new CityData {City = "Lisbon", Nation = "Portugal"},
                    new CityData {City = "Leningrad", Nation = "Russia"},
                    new CityData {City = "Moscow", Nation = "Russia"},
                    new CityData {City = "Tel Aviv", Nation = "Israel"},
                    new CityData {City = "Jerusalem", Nation = "Israel"},
                    new CityData {City = "Beijing", Nation = "China"},
                    new CityData {City = "Shanghai", Nation = "China"},
                    new CityData {City = "Hong Kong", Nation = "China"},
                    new CityData {City = "Zhengzhou", Nation = "China"},
                    new CityData {City = "Mumbai", Nation = "India"},
                    new CityData {City = "Hyderabad", Nation = "India"},
                    new CityData {City = "Ahmedabad", Nation = "India"},
                    new CityData {City = "Chennai", Nation = "India"},
                    new CityData {City = "Chennai", Nation = "India"},
                    new CityData {City = "Chennai", Nation = "India"},
                    new CityData {City = "Bangkok", Nation = "Thailand"},
                    new CityData {City = "Phnom Penh", Nation = "Cambodia"},
                    new CityData {City = "Ho Chi Minh City", Nation = "Vietnam"},
                    new CityData {City = "Seoul", Nation = "South Korea"},
                    new CityData {City = "Tokyo", Nation = "Japan"},
                    new CityData {City = "Nagano", Nation = "Japan"},
                    new CityData {City = "Osaka", Nation = "Japan"},
                    new CityData {City = "Vancouver", Nation = "Canada"},
                    new CityData {City = "Calgary", Nation = "Canada"},
                    new CityData {City = "Montreal", Nation = "Canada"},
                    new CityData {City = "Toronto", Nation = "Canada"},
                    new CityData {City = "Mexicy City", Nation = "Mexico"},
                    new CityData {City = "Rio de Janeiro", Nation = "Brazil"},
                    new CityData {City = "La Pz", Nation = "Bolivia"},
                    new CityData {City = "Lima", Nation = "Peru"},
                    new CityData {City = "Quito", Nation = "Ecuador"},
                    new CityData {City = "Bogota", Nation = "Columbia"},
                    new CityData {City = "Buenos Aires", Nation = "Argentina"},
                    new CityData {City = "Havana", Nation = "Cuba"},
                    new CityData {City = "Port au-Prince", Nation = "Haiti"},
                    new CityData {City = "Managua", Nation = "Nicaragua"},
                    new CityData {City = "Antigua", Nation = "Guatemala"},
                    new CityData {City = "San Jose", Nation = "Costa Rica"},
                    new CityData {City = "Reykjavik", Nation = "Iceland"},
                    new CityData {City = "Oslo", Nation = "Norway"},
                    new CityData {City = "Stockholm", Nation = "Sweden"},
                    new CityData {City = "Helsinki", Nation = "Finland"},
                    new CityData {City = "Tirana", Nation = "Albania"},
                    new CityData {City = "El-Biar", Nation = "Algeria"},
                    new CityData {City = "Budapest", Nation = "Hungary"},
                    new CityData {City = "Salzburg", Nation = "Austria"},
                    new CityData {City = "Sidney", Nation = "Australia"},
                    new CityData {City = "Perth", Nation = "Australia"},
                    new CityData {City = "Newcastle", Nation = "Australia"},
                    new CityData {City = "Timbu", Nation = "Bhutan"},
                    new CityData {City = "Fada", Nation = "Chad"},
                    new CityData {City = "Bangui", Nation = "Central African Republic"},
                    new CityData {City = "Ikelemba", Nation = "Congo"},
                    new CityData {City = "Auckland", Nation = "New Zealand"},
                    new CityData {City = "Christchurch", Nation = "New Zealand"},
                    new CityData {City = "Durban", Nation = "South Africa"},
                    new CityData {City = "Cape Town", Nation = "South Africa"},
                    new CityData {City = "Cairo", Nation = "Egypt"},
                    new CityData {City = "Amman", Nation = "Jordan"},
                    new CityData {City = "Kingston", Nation = "Jamaica"}
                };
        }

        private void InitSports()
        {
            if (this.sports != null)
                return;

            this.sports = new List<string>
                {
                    "Soccer",
                    "Soccer",
                    "Bandy",
                    "Baseball",
                    "Billiards",
                    "Boules",
                    "Bowling",
                    "Bridge",
                    "Chess",
                    "Cricket",
                    "Dance",
                    "Floorball",
                    "Golf",
                    "Karate",
                    "Korfball",
                    "Lifesaving",
                    "Motorcycle Racing",
                    "Mountaineering and Climbing",
                    "Netball",
                    "Orienteering",
                    "Pelota Vasca",
                    "Polo",
                    "Powerboating",
                    "Racquetball",
                    "Roller Sports",
                    "Rugby",
                    "Softball",
                    "Sport climbing",
                    "Squash",
                    "Sumo",
                    "Surfing",
                    "Tug of War",
                    "Underwater Sports",
                    "Water Ski",
                    "Wushu"
                };
        }

        private void Ds_InitializeMeasureDescriptor(object sender, InitializeMeasureDescriptorEventArgs e)
        {
            if (e.MeasureDescriptor.Name == "Budget")
            {
                e.MeasureDescriptor.DisplayFormat = "# ###.00";
                e.MeasureDescriptor.Aggregation = MeasureAggregation.Average;
            }
        }

        private void Ds_InitializeHierarchyDescriptor(object sender, InitializeHierarchyDescriptorEventArgs e)
        {
            LevelDescriptor level;
            if (e.HierarchyDescriptor.LevelDescriptors.TryGetItem("all", out level))
            {
                switch (e.HierarchyDescriptor.PropertyDescriptor.Name)
                {
                    case "Nation":
                        level.MemberProvider = item => "All Nations";
                        break;

                    case "City":
                        level.MemberProvider = item => "All Cities";
                        break;

                    case "Sport":
                        level.MemberProvider = item => "All Sports";
                        break;
                }
            }
        }

        private void Ds_AggregateMeasure(object sender, AggregateMeasureEventArgs e)
        {
            if (e.MeasureDescriptor.Name == "Budget")
            {
                double total = 0f;
                double count = 0f;
                foreach (object o in e.Items)
                {
                    var item = o as SportsData;
                    if (item != null)
                        total += item.Budget;
                    count += 1f;
                }

                e.Value = total / count;
                e.Handled = true;
            }
        }

        private void Ds_DataSourceError(object sender, Infragistics.Olap.DataSourceErrorEventArgs e)
        {
            DataSourceErrorReason reason = e.Reason;
            string message = e.Exception.Message;
            MessageBox.Show(message, reason.ToString());
        }

        private void Ds_InitializeAsyncCompleted(object sender, InitializeAsyncCompletedEventArgs e)
        {
            var ds = sender as FlatDataSource;

            if (ds != null)
            {
                ds.GetHierarchiesAsync(ultraPivotGrid1);
            }

            ds.ExpandAxisTupleMemberAsync(AxisType.Column, 0, 0);
        }

        private void Form1_Shown(object sender, EventArgs e)
        {

        }

        private void Form1_Load(object sender, EventArgs e)
        {
            var a = this.ultraPivotGrid1.CurrentState;
        }
    }
}
